#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "config.h"

unsigned char ports[] = { 4, 17, 27, 22, 10, 9, 11, 7 };
int size = sizeof(ports);

void init(int num)
{
  char path[255], exp[255], unexp[255];
  int i=0;
  FILE *fp=NULL;

  memset(exp,0,255);
  memset(unexp,0,255);
  snprintf(exp, 254, "%s/export", PFX);
  snprintf(unexp, 254, "%s/unexport", PFX);

  if (num<0) {
#ifdef DEBUG
    printf("de-init all\n");
#endif
    for (i=0; i<size; i++) {
      fp = fopen(unexp, "w");
      if (fp) {
        fprintf(fp, "%d\n", ports[i]);
        fclose(fp);
      }
    }
  }
  else {
#ifdef DEBUG
    printf("init %d\n", ports[num]);
#endif
    fp = fopen(exp, "w");
    if (fp) {
      fprintf(fp, "%d\n", ports[num]);
      fclose(fp);
    }

    memset(path,0,255);
    snprintf(path, 254, "%s/gpio%d/direction", PFX, ports[num]);

    fp = fopen(path, "w");
    if (fp) {
      fprintf(fp, "out\n");
      fclose(fp);
    }
  }
}

int gpio_set(int num, int msec)
{
  char vpath[255];
  FILE *fp=NULL;
  
  if ((num<0) || (num>size)) return(-1);

#ifdef DEBUG
  printf("setze %d (%d)\n", num, msec);
#endif

  snprintf(vpath, 254, "%s/gpio%d/value", PFX, ports[num]);
  fp = fopen(vpath, "w");
  if (fp) {
    fprintf(fp, "1\n"); 
    fflush(fp);

    if (msec>0) {
      usleep(msec * 1000);

      fprintf(fp, "0\n"); 
      fflush(fp);
    }

    fclose(fp);
  }

  return(0);
}

int gpio_clr(int num)
{
  char vpath[255];
  FILE *fp=NULL;
  
  if ((num<0) || (num>size)) return(-1);

#ifdef DEBUG
  printf("lösche %d\n", ports[num]);
#endif

  snprintf(vpath, 254, "%s/gpio%d/value", PFX, ports[num]);
  fp = fopen(vpath, "w");
  if (fp) {
    fprintf(fp, "0\n"); 
    fflush(fp);
    fclose(fp);
  }

  return(0);
}

int main(int argc, char **argv)
{
  int i=0;
  int num=-1;
  int mode=1; /* toggle=1, on=2, off=3 */

  if (argc>1) {
    num = atoi(argv[1]); /* number ? */

    if (strncmp(argv[1], "exit", 4)==0) {
      init(-1);
      return(0);
    }
    else if (strncmp(argv[1], "off", 3)==0) {
      for (i=0; i<size; i++) {
        init(i);
        gpio_clr(i);
      }
      return(0);
    }

    if (argc>2) {
      if (strncmp(argv[2], "on", 2)==0) mode=2;
      if (strncmp(argv[2], "off", 3)==0) mode=3;
    }
  }


  if ((num>=1) && (num<=8)) {

    init(num-1);

    switch(mode) {
      case 1:
        gpio_set(num-1,250);
        break;

      case 2:
        gpio_set(num-1,0);
        break;

      case 3:
        gpio_clr(num-1);
        break;
    }
  }

  return(0);
}
